// Copyright (C) 2019  Wilko Manger
//
// This file is part of matrix-dart-sdk.
//
// matrix-dart-sdk is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// matrix-dart-sdk is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
//
// You should have received a copy of the GNU Affero General Public License
// along with matrix-dart-sdk.  If not, see <https://www.gnu.org/licenses/>.

import 'package:matrix_sdk/matrix_sdk.dart';
import 'package:matrix_sdk/src/event/room/message_event.dart';
import 'package:matrix_sdk/src/event/room/state/avatar_change_event.dart';
import 'package:matrix_sdk/src/event/room/state/member_change_event.dart';
import 'package:matrix_sdk/src/event/room/state/name_change_event.dart';
import 'package:matrix_sdk/src/identifier.dart';

abstract class Event extends Identifiable<EventId> implements JsonWritable {
  String get _type;

  final EventId id;

  Event(this.id);

  factory Event.fromJson(RoomId roomId, Map<String, dynamic> json) {
    var type = json['type'];

    // Skip if it's not a valid event id
    if (!EventId.isValidFullyQualified(json['event_id'])) {
      return null;
    }

    // These might be null
    var id = EventId(json['event_id']);
    var sender = UserId(json['sender']);
    var stateKey = json['state_key'];

    var originServerTs = json['origin_server_ts'];
    var time;
    if (originServerTs != null) {
      time = DateTime.fromMillisecondsSinceEpoch(originServerTs);
    }

    switch (type) {
      case NameChangeEvent.type:
        return NameChangeEvent.fromJson(id, roomId, sender, time, json);
      case AvatarChangeEvent.type:
        return AvatarChangeEvent.fromJson(id, roomId, sender, time, json);
      case MemberChangeEvent.type:
        return MemberChangeEvent.fromJson(id, roomId, sender, time, stateKey, json);
      case MessageEvent.type:
        return MessageEvent.fromJson(id, roomId, sender, time, json);
      default:
        return null;
    }
  }

  @override
  Map<String, dynamic> toJson() => {
    'event_id': id.toJson(),
    'type': _type
  };
}