// Copyright (C) 2019  Wilko Manger
//
// This file is part of matrix-dart-sdk.
//
// matrix-dart-sdk is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// matrix-dart-sdk is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
//
// You should have received a copy of the GNU Affero General Public License
// along with matrix-dart-sdk.  If not, see <https://www.gnu.org/licenses/>.

import 'package:matrix_sdk/src/event/event.dart';
import 'package:matrix_sdk/src/event/room/message_event.dart';
import 'package:matrix_sdk/src/event/room/state/state_event.dart';
import 'package:matrix_sdk/src/storables.dart';
import 'package:matrix_sdk/src/identifier.dart';
import 'package:matrix_sdk/src/store.dart';

/// Events map that will lazily load from the given
/// [store].
class Events extends Storables<EventId, Event> {

  final RoomId _roomId;

  Events(this._roomId, {Store store}) : super(store: store);

  @override
  Future<void> save(EventId id, Event event)
    => store?.addEvent(_roomId, event);

  @override
  Future<Event> retrieve(EventId id) => store?.getEvent(_roomId, id);

  @override
  Stream<Event> retrieveAll() => store?.getEvents(_roomId);

  Stream<MessageEvent> messages() =>
      all().where((event) => event is MessageEvent).cast();

  Stream<StateEvent> state() =>
      all().where((event) => event is StateEvent).cast();
}
