// Copyright (C) 2019  Wilko Manger
//
// This file is part of matrix-dart-sdk.
//
// matrix-dart-sdk is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// matrix-dart-sdk is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
//
// You should have received a copy of the GNU Affero General Public License
// along with matrix-dart-sdk.  If not, see <https://www.gnu.org/licenses/>.

import 'package:matrix_sdk/matrix_sdk.dart';
import 'package:matrix_sdk/src/event/room/state/state_event.dart';

abstract class MemberChangeEvent extends StateEvent {
  static const String type = 'm.room.member';

  @override
  final String _type = type;

  /// The [UserId] of who has joined, left, was banned, etc.
  final UserId subject;

  /// Display name of the [subject]. Might be null.
  final String displayName;

  String get _membershipType;

  MemberChangeEvent(
      this.subject, this.displayName, EventId id,
      RoomId roomId, UserId sender, DateTime time)
      : super(id, roomId, sender, time, stateKey: subject.toString());

  factory MemberChangeEvent.fromJson(EventId id, RoomId roomId, UserId sender,
      DateTime time, String stateKey,  Map<String, dynamic> json) {
    final membershipType = json['content']['membership'];
    final subject = UserId(stateKey);
    final displayName = json['content']['displayname'];

    switch (membershipType) {
      case JoinEvent.type:
        return JoinEvent(subject, displayName, id, roomId, sender, time);
      case LeaveEvent.type:
        return LeaveEvent(subject, displayName, id, roomId, sender, time);
      case BanEvent.type:
        return BanEvent(subject, displayName, id, roomId, sender, time);
      case InviteEvent.type:
        return InviteEvent(subject, displayName, id, roomId, sender, time);
      case KnockEvent.type:
      default:
        return KnockEvent(subject, displayName, id, roomId, sender, time);
    }
  }

  @override
  Map<String, dynamic> toJson() => super.toJson()
    ..addAll({
      'content': {
        'membership': _membershipType
      }
    });
}

class JoinEvent extends MemberChangeEvent {
  static const String type = 'join';

  @override
  String get _membershipType => type;

  JoinEvent(
      UserId subject, String displayName, EventId id,
      RoomId roomId, UserId sender, DateTime time)
      : super(subject, displayName, id, roomId, sender, time);
}

class LeaveEvent extends MemberChangeEvent {
  static const String type = 'leave';

  @override
  String get _membershipType => type;

  LeaveEvent(
      UserId subject, String displayName, EventId id,
      RoomId roomId, UserId sender, DateTime time)
      : super(subject, displayName, id, roomId, sender, time);
}

class BanEvent extends MemberChangeEvent {
  static const String type = 'ban';

  @override
  String get _membershipType => type;

  BanEvent(
      UserId subject, String displayName, EventId id,
      RoomId roomId, UserId sender, DateTime time)
      : super(subject, displayName, id, roomId, sender, time);
}

// TODO: Add RoomState
class InviteEvent extends MemberChangeEvent {
  static const String type = 'invite';

  @override
  String get _membershipType => type;

  InviteEvent(
      UserId subject, String displayName, EventId id,
      RoomId roomId, UserId sender, DateTime time)
      : super(subject, displayName, id, roomId, sender, time);
}

class KnockEvent extends MemberChangeEvent {
  static const String type = 'knock';

  @override
  String get _membershipType => type;

  KnockEvent(
      UserId subject, String displayName, EventId id,
      RoomId roomId, UserId sender, DateTime time)
      : super(subject, displayName, id, roomId, sender, time);
}